@@ -5,6 +5,8 @@
 #ifndef QUERY_CAPACITY
 #define QUERY_CAPACITY
 
+#define QC_VERSION	"2.2.99"
+
 
 /* Build Customization */
 #define CONFIG_DEBUG_TIMESTAMPS		// Print timestamps in log
@@ -371,7 +373,7 @@
  * \b [1] Available starting with RHEL7.2 and SLES12SP1<br>
  * \b [2] <I>z/Architecture Principles of Operation</I>, SA22-7832<br>
  * \b [3] <I>z/VM: CP Commands and Utilities Reference</I>, SC24-6175<br>
- * \b [4] Requires global performance data to be enabled in the LPAR's activation profile<br>
+ * \b [4] Requires global performance data (GDP) to be enabled in the LPAR's activation profile<br>
  * \b [5] As of this writing, in LPARs with both CPs and IFLs defined in its activation profile, only CPs can become operational.
  *        Therefore, IFL counts would not appear in any of #qc_num_core_configured, #qc_num_core_standby, #qc_num_core_reserved,
  *        #qc_num_core_dedicated or #qc_num_core_shared
@@ -717,7 +719,7 @@ int qc_get_num_layers(void *hdl, int *rc);
  * - 1: LPAR layer information, etc.
  * @param value Return parameter returning the string attribute's value or NULL
  * in case of an error.
- @return Indicating validity of the queried attribute as follows:
+ * @return Indicating validity of the queried attribute as follows:
  * - >0  attribute is valid
  * -  0  attribute exists but is not set
  * - <0  an error occurred retrieving the attribute
@@ -739,7 +741,7 @@ int qc_get_attribute_string(void *hdl, enum qc_attr_id id, int layer, const char
  * - 1: LPAR layer information, etc.
  * @param value Return parameter returning the string attribute's value or undefined
  * in case of an error.
- @return Indicating validity of the queried attribute as follows:
+ * @return Indicating validity of the queried attribute as follows:
  * - >0  attribute is valid
  * -  0  attribute exists but is not set
  * - <0  an error occurred retrieving the attribute
@@ -762,11 +764,17 @@ int qc_get_attribute_int(void *hdl, enum qc_attr_id id, int layer, int *value);
  * - 1: LPAR layer information, etc.
  * @param value Return parameter returning the float attribute's value or undefined
  * in case of an error.indicating validity as follows:
- @return Indicating validity of the queried attribute as follows:
+ * @return Indicating validity of the queried attribute as follows:
  * - >0  attribute is valid
  * -  0  attribute exists but is not set
  * - <0  an error occurred retrieving the attribute
  */
 int qc_get_attribute_float(void *hdl, enum qc_attr_id id, int layer, float *value);
 
+/**
+ * Prints the internal data in JSON format to stdout.
+ * @param hdl Handle of the configuration to use.
+ */
+void qc_export_json(void *hdl);
+
 #endif
