@@ -1,4 +1,4 @@
-/* Copyright IBM Corp. 2013, 2019 */
+/* Copyright IBM Corp. 2013, 2020 */
 
 #include "query_capacity_data.h"
 
@@ -122,8 +122,8 @@ struct qc_lpar {
 	char layer_extended_name[QC_LEN_LAYER_EXTENDED_NAME];
 	char layer_uuid[QC_LEN_LAYER_UUID];
 	int adjustment;
-        int has_secure;
-        int secure;
+	int has_secure;
+	int secure;
 	int num_core_total;
 	int num_core_configured;
 	int num_core_standby;
@@ -714,17 +714,17 @@ const char *qc_attr_id_to_char(struct qc_handle *hdl, enum qc_attr_id id) {
 	case qc_num_core_reserved: return "num_core_reserved";
 	case qc_num_core_dedicated: return "num_core_dedicated";
 	case qc_num_core_shared: return "num_core_shared";
-	case qc_ziip_absolute_capping: return "qc_ziip_absolute_capping";
-	case qc_ziip_capacity_cap: return "qc_ziip_capacity_cap";
-	case qc_ziip_capped_capacity: return "qc_ziip_capped_capacity";
-	case qc_ziip_dispatch_limithard: return "qc_ziip_dispatch_limithard";
-	case qc_ziip_dispatch_type: return "qc_ziip_dispatch_type";
-	case qc_ziip_limithard_cap: return "qc_ziip_limithard_cap";
-	case qc_ziip_weight_capping: return "qc_ziip_weight_capping";
-	case qc_num_ziip_dedicated: return "qc_num_ziip_dedicated";
-	case qc_num_ziip_shared: return "qc_num_ziip_shared";
-	case qc_num_ziip_total: return "qc_num_ziip_total";
-	case qc_num_ziip_threads: return "qc_num_ziip_threads";
+	case qc_ziip_absolute_capping: return "ziip_absolute_capping";
+	case qc_ziip_capacity_cap: return "ziip_capacity_cap";
+	case qc_ziip_capped_capacity: return "ziip_capped_capacity";
+	case qc_ziip_dispatch_limithard: return "ziip_dispatch_limithard";
+	case qc_ziip_dispatch_type: return "ziip_dispatch_type";
+	case qc_ziip_limithard_cap: return "ziip_limithard_cap";
+	case qc_ziip_weight_capping: return "ziip_weight_capping";
+	case qc_num_ziip_dedicated: return "num_ziip_dedicated";
+	case qc_num_ziip_shared: return "num_ziip_shared";
+	case qc_num_ziip_total: return "num_ziip_total";
+	case qc_num_ziip_threads: return "num_ziip_threads";
 	default: break;
 	}
 	qc_debug(hdl, "Error: Cannot convert unknown attribute '%d' to char*\n", id);
@@ -732,8 +732,18 @@ const char *qc_attr_id_to_char(struct qc_handle *hdl, enum qc_attr_id id) {
 	return NULL;
 }
 
+struct qc_handle *qc_hdl_get_cec(struct qc_handle *hdl) {
+	return hdl ? hdl->root : hdl;
+}
+
+struct qc_handle *qc_hdl_get_lpar(struct qc_handle *hdl) {
+	for (hdl = hdl->root; hdl != NULL && *(int *)(hdl->layer) != QC_LAYER_TYPE_LPAR; hdl = hdl->next);
+
+	return hdl;
+}
+
 // 'hdl' is for error reporting, as 'tgthdl' might not be part of the pointer lists yet
-int qc_new_handle(struct qc_handle *hdl, struct qc_handle **tgthdl, int layer_no,
+int qc_hdl_new(struct qc_handle *hdl, struct qc_handle **tgthdl, int layer_no,
 		  int layer_type_num) {
 	int num_attrs, layer_category_num;
 	char *layer_type, *layer_category;
@@ -823,7 +833,7 @@ int qc_new_handle(struct qc_handle *hdl, struct qc_handle **tgthdl, int layer_no
 		layer_type = "z/OS-zCX-Server";
 		break;
 	default:
-		qc_debug(hdl, "Error: Unhandled layer type in qc_new_handle()\n");
+		qc_debug(hdl, "Error: Unhandled layer type in qc_hdl_new()\n");
 		return -1;
 	}
 
@@ -874,12 +884,12 @@ int qc_new_handle(struct qc_handle *hdl, struct qc_handle **tgthdl, int layer_no
 	return 0;
 }
 
-int qc_insert_handle(struct qc_handle *hdl, struct qc_handle **inserted_hdl, int type) {
-	struct qc_handle *prev_hdl = qc_get_prev_handle(hdl);
+int qc_hdl_insert(struct qc_handle *hdl, struct qc_handle **inserted_hdl, int type) {
+	struct qc_handle *prev_hdl = qc_hdl_get_prev(hdl);
 
 	if (!prev_hdl)
 		return -1;
-	if (qc_new_handle(hdl, inserted_hdl, hdl->layer_no, type))
+	if (qc_hdl_new(hdl, inserted_hdl, hdl->layer_no, type))
 		return -2;
 	(*inserted_hdl)->next = hdl;
 	prev_hdl->next = *inserted_hdl;
@@ -890,10 +900,41 @@ int qc_insert_handle(struct qc_handle *hdl, struct qc_handle **inserted_hdl, int
 	return 0;
 }
 
-int qc_append_handle(struct qc_handle *hdl, struct qc_handle **appended_hdl, int type) {
+void qc_hdl_prune(struct qc_handle *hdl) {
+        struct qc_handle *ptr = hdl, *skip = NULL;
+
+        // Pruning at the root needs special handling as does intermediate pruning
+        if (hdl == qc_hdl_get_root(hdl))
+                skip = hdl;
+        else {
+                hdl = qc_hdl_get_prev(hdl);
+                hdl->next = NULL;
+        }
+
+        while (ptr) {
+		free(ptr->layer);
+		free(ptr->attr_present);
+		free(ptr->src);
+		hdl = ptr->next;
+		if (ptr == skip) {
+			memset(ptr, 0, sizeof(struct qc_handle));
+			ptr->root = ptr;
+		} else
+			free(ptr);
+		ptr = hdl;
+	}
+
+	return;
+}
+
+int qc_hdl_get_layer_no(struct qc_handle *hdl) {
+        return hdl->layer_no;
+}
+
+int qc_hdl_append(struct qc_handle *hdl, struct qc_handle **appended_hdl, int type) {
 	struct qc_handle *next_hdl = hdl->next;
 
-	if (qc_new_handle(hdl, appended_hdl, hdl->layer_no + 1, type))
+	if (qc_hdl_new(hdl, appended_hdl, hdl->layer_no + 1, type))
 		return -1;
 	hdl->next = *appended_hdl;
 	(*appended_hdl)->next = next_hdl;
@@ -1087,17 +1128,17 @@ int qc_is_attr_set_string(struct qc_handle *hdl, enum qc_attr_id id) {
 	return qc_is_attr_set(hdl, id, string);
 }
 
-struct qc_handle *qc_get_root_handle(struct qc_handle *hdl) {
+struct qc_handle *qc_hdl_get_root(struct qc_handle *hdl) {
 	return hdl ? hdl->root : NULL;
 }
 
-struct qc_handle *qc_get_top_handle(struct qc_handle *hdl) {
+struct qc_handle *qc_hdl_get_top(struct qc_handle *hdl) {
 	for (; hdl->next != NULL; hdl = hdl->next);
 
 	return hdl;
 }
 
-struct qc_handle *qc_get_prev_handle(struct qc_handle *hdl) {
+struct qc_handle *qc_hdl_get_prev(struct qc_handle *hdl) {
 	struct qc_handle *prev_hdl = NULL;
 
 	for (prev_hdl = hdl->root; prev_hdl->next != NULL; prev_hdl = prev_hdl->next)
@@ -1168,3 +1209,27 @@ char qc_get_attr_value_src_float(struct qc_handle *hdl, enum qc_attr_id id) {
 char qc_get_attr_value_src_string(struct qc_handle *hdl, enum qc_attr_id id) {
 	return qc_get_attr_value_src(hdl, id, string);
 }
+
+void qc_print_attrs_json(struct qc_handle *hdl, int indent) {
+        struct qc_attr *attr;
+        void *val;
+
+        for (attr = hdl->attr_list; attr->offset >= 0; attr++) {
+                if ((val = qc_get_attr_value(hdl, attr->id, attr->type)) == NULL)
+                        printf("%*s\"%s\": null", indent, "", qc_attr_id_to_char(hdl, attr->id));
+                else {
+                        switch (attr->type) {
+                        case integer:
+                                printf("%*s\"%s\": \"%d\"", indent, "", qc_attr_id_to_char(hdl, attr->id), *(int*)val);
+                                break;
+                        case floatingpoint:
+                                printf("%*s\"%s\": \"%f\"", indent, "", qc_attr_id_to_char(hdl, attr->id), *(float*)val);
+                                break;
+                        case string:
+                                printf("%*s\"%s\": \"%s\"", indent, "", qc_attr_id_to_char(hdl, attr->id), (char*)val);
+                                break;
+                        }
+                }
+                printf("%s\n", (attr + 1)->offset >= 0 ? "," : "");
+        }
+}
