@@ -1,18 +1,15 @@
-/* Copyright IBM Corp. 2013, 2017 */
+/* Copyright IBM Corp. 2013, 2019 */
 
 #define _GNU_SOURCE
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/param.h>
 #include <fcntl.h>
 #include <string.h>
-#include <unistd.h>
 
-#include "query_capacity_int.h"
 #include "query_capacity_data.h"
 
 
-#define MAX(a,b) ((a)>(b)?a:b)
-
 static const char *qc_sysinfo_delim = "\n";
 
 
@@ -127,12 +124,12 @@ static int qc_sysinfo_lgm_check(struct qc_handle *hdl, const char *sysinfo) {
 	qc_debug_indent_inc();
 	if (qc_sysinfo_open(hdl, &lsysinfo)) {
 		qc_debug(hdl, "Error: Failed to open /proc/sysinfo\n");
-		rc = 1;
+		rc = -1;
 		goto out;
 	}
 	if (strcmp(lsysinfo, sysinfo)) {
-		qc_debug(hdl, "/proc/sysinfo content changed, LGM took place\n");
-		rc = 2;
+		qc_debug(hdl, "/proc/sysinfo content changed, LGM took place!\n");
+		rc = 1;
 		goto out;
 	}
 	qc_debug(hdl, "/proc/sysinfo still consistent, no LGM detected\n");
@@ -161,24 +158,25 @@ static char *qc_copy_sysinfo(struct qc_handle *hdl, char *sysinfo) {
 }
 
 #define QC_SYSINFO_PARSE_LINE_STR_NOCONT(hdl, str, strlen, id) \
-	if (sscanf(*line, str, str_buf) > 0 && qc_set_attr_string(hdl, id, str_buf, strlen, ATTR_SRC_SYSINFO)) \
-		goto out;
+	if (sscanf(*line, str, str_buf) > 0 && qc_set_attr_string(hdl, id, str_buf, ATTR_SRC_SYSINFO)) \
+		goto out_err;
 #define QC_SYSINFO_PARSE_LINE_STR(hdl, str, strlen, id) \
 	if (sscanf(*line, str, str_buf) > 0) { \
-		if (qc_set_attr_string(hdl, id, str_buf, strlen, ATTR_SRC_SYSINFO)) \
-			goto out; \
+		if (qc_set_attr_string(hdl, id, str_buf, ATTR_SRC_SYSINFO)) \
+			goto out_err; \
 		continue; \
 	}
-#define QC_SYSINFO_PARSE_LINE_INT(hdl, str, id) \
+#define QC_SYSINFO_PARSE_LINE_INT_inc(hdl, str, id, inc) \
 	if (sscanf(*line, str, &int_buf) > 0) { \
-		if (qc_set_attr_int(hdl, id, int_buf, ATTR_SRC_SYSINFO)) \
-			goto out; \
+		if (qc_set_attr_int(hdl, id, int_buf + inc, ATTR_SRC_SYSINFO)) \
+			goto out_err; \
 		continue; \
 	}
+#define QC_SYSINFO_PARSE_LINE_INT(hdl, str, id) QC_SYSINFO_PARSE_LINE_INT_inc(hdl, str, id, 0)
 #define QC_SYSINFO_PARSE_LINE_FLOAT(hdl, str, id) \
 	if (sscanf(*line, str, &float_buf)) { \
 		if (qc_set_attr_float(hdl, id, float_buf, ATTR_SRC_SYSINFO)) \
-			goto out; \
+			goto out_err; \
 		continue; \
 	}
 static int qc_fill_in_sysinfo_values_vm(struct qc_handle *hdl, char **sptr, char **line) {
@@ -224,6 +222,12 @@ static int qc_fill_in_sysinfo_values_vm(struct qc_handle *hdl, char **sptr, char
 			guesttype = QC_LAYER_TYPE_ZVM_GUEST;
 			qc_debug(hdl, "Layer %2d: z/VM-host\n", hdl->layer_no + 1);
 			qc_debug(hdl, "Layer %2d: z/VM-guest\n", hdl->layer_no + 2);
+		} else if (!strncmp(str_buf, "z/OS zCX", strlen("z/OS zCX")) ||
+			   !strncmp(str_buf, "KVM zCX", strlen("KVM zCX"))) {
+			hosttype = QC_LAYER_TYPE_ZOS_HYPERVISOR;
+			guesttype = QC_LAYER_TYPE_ZOS_ZCX_SERVER;
+			qc_debug(hdl, "Layer %2d: z/OS-host\n", hdl->layer_no + 1);
+			qc_debug(hdl, "Layer %2d: z/OS-zCX-Server\n", hdl->layer_no + 2);
 		} else if (!strncmp(str_buf, "KVM", strlen("KVM"))) {
 			hosttype = QC_LAYER_TYPE_KVM_HYPERVISOR;
 			guesttype = QC_LAYER_TYPE_KVM_GUEST;
@@ -235,16 +239,16 @@ static int qc_fill_in_sysinfo_values_vm(struct qc_handle *hdl, char **sptr, char
 			goto out;
 		}
 		if (!hdl->next)
-			rc = qc_append_handle(hdl, &hosthdl, hosttype);
+			rc = qc_hdl_append(hdl, &hosthdl, hosttype);
 		else
-			rc = qc_insert_handle(hdl->next, &hosthdl, hosttype);
+			rc = qc_hdl_insert(hdl->next, &hosthdl, hosttype);
 		if (rc)
 			goto out;
-		if (qc_append_handle(hosthdl, &guesthdl, guesttype))
-			goto out;
-		if (qc_set_attr_string(hosthdl, qc_control_program_id, str_buf, 16, ATTR_SRC_SYSINFO) ||
-		    qc_set_attr_string(guesthdl, qc_layer_name, layer_name, 8, ATTR_SRC_SYSINFO))
-			goto out;
+		if (qc_hdl_append(hosthdl, &guesthdl, guesttype))
+			goto out_err;
+		if (qc_set_attr_string(hosthdl, qc_control_program_id, str_buf, ATTR_SRC_SYSINFO) ||
+		    qc_set_attr_string(guesthdl, qc_layer_name, layer_name, ATTR_SRC_SYSINFO))
+			goto out_err;
 		for (sysi = NULL, *line = strtok_r(sysi, qc_sysinfo_delim, sptr); *line;
 		     *line = strtok_r(sysi, qc_sysinfo_delim, sptr)) {
 			if (strncmp(*line, vmxx, 4) != 0)
@@ -266,7 +270,10 @@ static int qc_fill_in_sysinfo_values_vm(struct qc_handle *hdl, char **sptr, char
 		}
 	}
 	rc = 0;
+	goto out;
 
+out_err:
+	rc = -1;
 out:
 	qc_debug_indent_dec();
 
@@ -316,7 +323,7 @@ out:
 
 static int qc_fill_in_sysinfo_values_lpar(struct qc_handle *hdl, char **sptr, char **line) {
 	char *sysi = NULL, str_buf[STR_BUF_SIZE];	// large enough for all sscanf() calls by far
-	int int_buf, rc = -1;
+	int int_buf, rc = -1, ps_mtid = -1, *i;
 
 	qc_debug(hdl, "Retrieve /proc/sysinfo information for LPAR\n");
 	qc_debug_indent_inc();
@@ -331,18 +338,34 @@ static int qc_fill_in_sysinfo_values_lpar(struct qc_handle *hdl, char **sptr, ch
 		QC_SYSINFO_PARSE_LINE_INT(hdl, "Adjustment: %i", qc_adjustment);
 		if (strncmp(*line, "CPUs ", 5) == 0) {
 			*line += 5;
-			QC_SYSINFO_PARSE_LINE_INT(hdl, "Total: %i", qc_num_cpu_total);
-			QC_SYSINFO_PARSE_LINE_INT(hdl, "Configured: %i", qc_num_cpu_configured);
-			QC_SYSINFO_PARSE_LINE_INT(hdl, "Standby: %i", qc_num_cpu_standby);
-			QC_SYSINFO_PARSE_LINE_INT(hdl, "Reserved: %i", qc_num_cpu_reserved);
-			QC_SYSINFO_PARSE_LINE_INT(hdl, "Dedicated: %i", qc_num_cpu_dedicated);
-			QC_SYSINFO_PARSE_LINE_INT(hdl, "Shared: %i", qc_num_cpu_shared);
+			QC_SYSINFO_PARSE_LINE_INT(hdl, "Total: %i", qc_num_core_total);
+			QC_SYSINFO_PARSE_LINE_INT(hdl, "Configured: %i", qc_num_core_configured);
+			QC_SYSINFO_PARSE_LINE_INT(hdl, "Standby: %i", qc_num_core_standby);
+			QC_SYSINFO_PARSE_LINE_INT(hdl, "Reserved: %i", qc_num_core_reserved);
+			QC_SYSINFO_PARSE_LINE_INT(hdl, "Dedicated: %i", qc_num_core_dedicated);
+			QC_SYSINFO_PARSE_LINE_INT(hdl, "Shared: %i", qc_num_core_shared);
+			QC_SYSINFO_PARSE_LINE_INT_inc(hdl, "G-MTID: %i", qc_num_cp_threads, 1);
+			QC_SYSINFO_PARSE_LINE_INT_inc(hdl, "S-MTID: %i", qc_num_ifl_threads, 1);
+			if (sscanf(*line, "PS-MTID: %i", &int_buf) > 0)
+				ps_mtid = int_buf + 1;
 		}
 		QC_SYSINFO_PARSE_LINE_STR(hdl, "Extended Name: %256[^\n]", 256, qc_layer_extended_name);
 		QC_SYSINFO_PARSE_LINE_STR(hdl, "UUID: %36s", 36, qc_layer_uuid);
 	}
 	rc = qc_derive_part_char_num(hdl);
+	// Apply threshold provided by ps_mtid if set
+	if (ps_mtid >= 0) {
+		qc_debug(hdl, "Apply PS-MTID limit of %d\n", ps_mtid);
+		if ((i = qc_get_attr_value_int(hdl, qc_num_cp_threads)) != NULL)
+			*i = MIN(ps_mtid, *i);
+		if ((i = qc_get_attr_value_int(hdl, qc_num_ifl_threads)) != NULL)
+			*i = MIN(ps_mtid, *i);
+	}
 
+	goto out;
+
+out_err:
+	rc = -1;
 out:
 	qc_debug_indent_dec();
 
@@ -358,8 +381,8 @@ static int qc_fill_in_sysinfo_values_cec(struct qc_handle *hdl, char **sptr, cha
 	qc_debug_indent_inc();
 	if (qc_set_attr_int(hdl, qc_layer_type_num, QC_LAYER_TYPE_CEC, ATTR_SRC_SYSINFO) ||
 	    qc_set_attr_int(hdl, qc_layer_category_num, QC_LAYER_CAT_HOST, ATTR_SRC_SYSINFO) ||
-	    qc_set_attr_string(hdl, qc_layer_type, "CEC", sizeof("CEC"), ATTR_SRC_SYSINFO) ||
-	    qc_set_attr_string(hdl, qc_layer_category, "HOST", sizeof("HOST"), ATTR_SRC_SYSINFO))
+	    qc_set_attr_string(hdl, qc_layer_type, "CEC", ATTR_SRC_SYSINFO) ||
+	    qc_set_attr_string(hdl, qc_layer_category, "HOST", ATTR_SRC_SYSINFO))
 		goto out;
 	for (; *line && strncmp(*line, "LPAR", 4); *line = strtok_r(sysi, qc_sysinfo_delim, sptr)) {
 		sysi = NULL;
@@ -367,6 +390,7 @@ static int qc_fill_in_sysinfo_values_cec(struct qc_handle *hdl, char **sptr, cha
 			continue;	// Lots of "Adjustment" lines that we can skip
 		QC_SYSINFO_PARSE_LINE_STR(hdl, "Manufacturer: %16s", 16, qc_manufacturer);
 		QC_SYSINFO_PARSE_LINE_STR(hdl, "Type: %4s", 4, qc_type);
+		QC_SYSINFO_PARSE_LINE_STR(hdl, "LIC Identifier: %16s", 16, qc_lic_identifier);
 		if (strncmp(*line, "Model: ", 7) == 0) {
 			*line += 7;
 			QC_SYSINFO_PARSE_LINE_STR_NOCONT(hdl, "%16s", 16, qc_model_capacity);
@@ -382,16 +406,21 @@ static int qc_fill_in_sysinfo_values_cec(struct qc_handle *hdl, char **sptr, cha
 		}
 		if (strncmp(*line, "CPUs ", 5) == 0) {
 			*line += 5;
-			QC_SYSINFO_PARSE_LINE_INT(hdl, "Total: %i", qc_num_cpu_total);
-			QC_SYSINFO_PARSE_LINE_INT(hdl, "Configured: %i", qc_num_cpu_configured);
-			QC_SYSINFO_PARSE_LINE_INT(hdl, "Standby: %i", qc_num_cpu_standby);
-			QC_SYSINFO_PARSE_LINE_INT(hdl, "Reserved: %i", qc_num_cpu_reserved);
+			QC_SYSINFO_PARSE_LINE_INT(hdl, "Total: %i", qc_num_core_total);
+			QC_SYSINFO_PARSE_LINE_INT(hdl, "Configured: %i", qc_num_core_configured);
+			QC_SYSINFO_PARSE_LINE_INT(hdl, "Standby: %i", qc_num_core_standby);
+			QC_SYSINFO_PARSE_LINE_INT(hdl, "Reserved: %i", qc_num_core_reserved);
+			QC_SYSINFO_PARSE_LINE_INT_inc(hdl, "G-MTID: %i", qc_num_cp_threads, 1);
+			QC_SYSINFO_PARSE_LINE_INT_inc(hdl, "S-MTID: %i", qc_num_ifl_threads, 1);
 		}
 		QC_SYSINFO_PARSE_LINE_FLOAT(hdl, "Capability: %f", qc_capability);
 		QC_SYSINFO_PARSE_LINE_FLOAT(hdl, "Secondary Capability: %f", qc_secondary_capability);
 	}
 	rc = 0;
+	goto out;
 
+out_err:
+	rc = -1;
 out:
 	qc_debug_indent_dec();
 
@@ -399,7 +428,7 @@ out:
 }
 
 static int qc_sysinfo_process(struct qc_handle *hdl, char *sysinfo) {
-	struct qc_handle *lparhdl = qc_get_lpar_handle(hdl);
+	struct qc_handle *lparhdl = qc_hdl_get_lpar(hdl);
 	char *sysi = NULL, *start, *sptr, *line;
 	int rc = -1;
 
